// ignore_for_file: unnecessary_null_comparison

import 'package:rec_api_dart/src/entities/bank_card.ent.dart';

import '../helpers/checks.dart';
import 'activity.ent.dart';
import 'badge.ent.dart';
import 'campaign.ent.dart';
import 'entity.base.dart';
import 'formatted_address.dart';
import 'level.ent.dart';
import 'offer.ent.dart';
import 'product.ent.dart';
import 'role.ent.dart';
import 'schedule/schedule.ent.dart';
import 'transactions/currency.ent.dart';
import 'transactions/wallet.ent.dart';

/// This entity hold the information for an [Account]
class Account extends Entity {
  static const String TYPE_PRIVATE = 'PRIVATE';
  static const String TYPE_COMPANY = 'COMPANY';

  static const String SUBTYPE_COMPANY_RETAILER = 'RETAILER';
  static const String SUBTYPE_COMPANY_WHOLESALE = 'WHOLESALE';

  static const String SUBTYPE_COMPANY_NORMAL = 'NORMAL';
  static const String SUBTYPE_COMPANY_BMINCOME = 'BMINCOME';

  static const String REZERO_B2B_ACCESS_NOT_GRANTED = 'not_granted';
  static const String REZERO_B2B_ACCESS_PENDING = 'pending';
  static const String REZERO_B2B_ACCESS_GRANTED = 'granted';

  List<Activity>? activities = [];
  List<Product>? producingProducts = [];
  List<Product>? consumingProducts = [];
  List<Wallet> wallets = [];
  List<Campaign> campaigns = [];
  List<Offer>? offers = [];
  List<Role> roles = [];
  List<Badge> badges = [];
  List<BankCard> bankCards = [];

  String? name;
  String? companyImage;
  String? publicImage;
  String? type;
  String? recAddress;
  String? description;
  String? webUrl;
  String? email;
  String? phone;
  String? prefix;

  Level? level;
  Schedule? schedule;
  FormattedAddress? address;

  String? scheduleString;
  String? addressString;
  String? rezeroB2BUsername;
  String? rezeroB2BAccess;

  double? latitude;
  double? longitude;

  num? redeemableAmount;
  num? rewardedAmount;

  bool? active;
  bool? hasOffers;
  bool? isInLtabCampaign;
  bool hasSavedCards;

  Account({
    String? id,
    String? createdAt,
    String? updatedAt,
    this.name,
    this.active,
    this.type,
    this.activities,
    this.consumingProducts,
    this.producingProducts,
    this.companyImage,
    this.publicImage,
    this.wallets = const [],
    this.campaigns = const [],
    this.roles = const [],
    this.badges = const [],
    this.bankCards = const [],
    this.recAddress,
    this.hasSavedCards = false,
    this.description,
    this.schedule,
    this.latitude,
    this.longitude,
    this.addressString,
    this.address,
    this.level,
    this.prefix,
    this.phone,
    this.webUrl,
    this.email,
    this.offers,
    this.scheduleString,
    this.redeemableAmount,
    this.rewardedAmount,
    this.rezeroB2BUsername,
    this.rezeroB2BAccess,
    this.isInLtabCampaign = false,
    this.hasOffers,
  }) : super(id, createdAt, updatedAt);

  factory Account.fromJson(Map<String, dynamic> json) {
    var wallets = <Wallet>[];
    var campaigns = <Campaign>[];
    var offers = <Offer>[];
    var roles = <Role>[];
    var badges = <Badge>[];
    var bankCards = <BankCard>[];

    if (json['wallets'] != null) {
      final jsonWallets = List.from(json['wallets']);
      wallets = jsonWallets.isNotEmpty
          ? jsonWallets.map<Wallet>((el) => Wallet.fromJson(el)).toList()
          : <Wallet>[];
    }

    if (json['campaigns'] != null) {
      final jsonCampaigns = List.from(json['campaigns']);
      try {
        campaigns = jsonCampaigns.isNotEmpty
            ? jsonCampaigns.map<Campaign>((el) => Campaign.fromJson(el)).toList()
            : <Campaign>[];
      } catch (e) {
        // TODO: remove in next big version
      }
    }

    if (json['offers'] != null) {
      final jsonOffers = List.from(json['offers']);
      offers = jsonOffers.isNotEmpty
          ? jsonOffers.map<Offer>((el) => Offer.fromJson(el)).toList()
          : <Offer>[];
    }

    if (json['roles'] != null) {
      final jsonRoles = List.from(json['roles']);
      roles = jsonRoles.isNotEmpty
          ? jsonRoles.map<Role>((el) => Role.getRoleByName(el)).toList()
          : <Role>[];
    }

    if (json['badges'] != null) {
      final jsonBadges = List.from(json['badges']);
      badges = jsonBadges.isNotEmpty
          ? jsonBadges.map<Badge>((el) => Badge.fromJson(el)).toList()
          : <Badge>[];
    }

    if (json['bank_cards'] != null) {
      final bankCardsJson = List.from(json['bank_cards']);
      bankCards = bankCardsJson.isNotEmpty
          ? bankCardsJson.map<BankCard>((el) => BankCard.fromJson(el)).toList()
          : <BankCard>[];
    }

    final formattedAddress = FormattedAddress.fromJson(json);
    final addressString = formattedAddress.toString();

    return Account(
      id: '${json['id']}',
      createdAt: json['created'],
      updatedAt: json['updated'],
      name: json['name'] == null || json['name'].isEmpty ? 'PARTICULAR' : json['name'],
      type: json['type'],
      active: json['active'],
      publicImage: json['public_image'],
      companyImage: json['company_image'],
      recAddress: json['rec_address'],
      description: json['description'],
      schedule: Schedule.fromJsonString(json['schedule']),
      scheduleString: json['schedule'],
      prefix: json['prefix'],
      phone: json['phone'],
      email: json['email'],
      latitude: double.parse((json['latitude'] ?? 0).toString()),
      longitude: double.parse((json['longitude'] ?? 0).toString()),
      redeemableAmount: json['redeemable_amount'],
      rewardedAmount: json['rewarded_amount'],
      level: Checks.isNotNull(json['level']) ? Level.fromJson(json['level']) : null,
      wallets: wallets,
      campaigns: campaigns,
      roles: roles,
      badges: badges,
      bankCards: bankCards,
      offers: offers..sort((a, b) => b.updatedAt.compareTo(a.updatedAt)),
      webUrl: json['web'],
      addressString: addressString,
      address: formattedAddress,
      hasOffers: json['has_offers'] ?? false,
      hasSavedCards: json['has_saved_cards'] ?? false,
      isInLtabCampaign: json['in_ltab_campaign'],
      rezeroB2BUsername: json['rezero_b2b_username'],
      rezeroB2BAccess: json['rezero_b2b_access'],
    );
  }

  /// Returns the active offers
  List<Offer>? get activeOffers {
    if (offers == null || offers!.isEmpty) return [];
    return offers!.where((element) => element.isActive).toList();
  }

  String get fullPhone {
    if (prefix == null || prefix!.isEmpty || prefix == '+') {
      return '$phone';
    }

    return '+$prefix $phone';
  }

  bool belongsToCampaign(String? campaignCode) {
    return campaigns.any((campaign) => campaign.code == campaignCode);
  }

  Wallet getWallet(String currencyName) {
    return wallets.firstWhere((element) => element.currency!.name == currencyName);
  }

  int? getWalletBalance(String currencyName) {
    return wallets.firstWhere((element) => element.currency!.name == currencyName).balance;
  }

  int? getWalletBalanceByCurrency(Currency currency) {
    return getWalletBalance(currency.name);
  }

  Wallet getWalletByCurrency(Currency currency) {
    return wallets.firstWhere((element) => element.currency == currency);
  }

  bool hasLevel(Level level) {
    return level == this.level;
  }

  bool hasLevelByCode(String code) {
    return level != null && level!.code == code;
  }

  bool hasPublicImage() {
    return Checks.isNotEmpty(publicImage);
  }

  bool hasWeb() {
    return Checks.isNotEmpty(webUrl);
  }

  bool isCampaignAccount(String? campaignCode) {
    return isPrivate() && !Checks.isEmpty(campaigns) && belongsToCampaign(campaignCode);
  }

  bool isCampaignActive(Campaign campaign) {
    return campaigns != null && campaigns.contains(campaign);
  }

  bool isCampaignActiveById(String campaignId) {
    return campaigns != null &&
        campaigns.isNotEmpty &&
        campaigns.firstWhere((el) => el.id == campaignId) != null;
  }

  bool isCompany() {
    return type == Account.TYPE_COMPANY;
  }

  // CAMPAIGNS
  bool isLtabAccount({String ltabAccountName = 'LI TOCA AL BARRI'}) {
    return name == ltabAccountName;
  }

  bool isPrivate() {
    return type == Account.TYPE_PRIVATE;
  }
}
