import 'package:rec_api_dart/src/entities/entity.base.dart';

class ConfigurationSetting {
  final String scope;
  final String name;
  final String value;

  ConfigurationSetting({
    required this.scope,
    required this.name,
    required this.value,
  });

  factory ConfigurationSetting.fromJson(Map<String, dynamic> json) {
    return ConfigurationSetting(
      name: json['name'],
      scope: json['scope'],
      value: json['value'],
    );
  }
}

/// Hold the configuration settings os the current environment
/// Used to enable/disable certain features
///
/// For more info check the [ConfigurationSettings Docs](https://github.com/QbitArtifacts/rec_api_dart/wiki/ConfigurationSettings)
class ConfigurationSettings extends Entity {
  final List<ConfigurationSetting> settings;
  final Map<String, ConfigurationSetting> settingsMap;

  ConfigurationSettings({
    String? id,
    String? createdAt,
    String? updatedAt,
    this.settings = const [],
    this.settingsMap = const {},
  }) : super(id, createdAt, updatedAt);

  factory ConfigurationSettings.fromJson(Map<String, dynamic> json) {
    List<ConfigurationSetting> settings = [];
    Map<String, ConfigurationSetting> settingsMap = {};

    if (json['elements'] != null) {
      final elements = List.from(json['elements']);
      settings = elements.isNotEmpty
          ? elements.map((el) => ConfigurationSetting.fromJson(el)).toList()
          : <ConfigurationSetting>[];
    }

    for (final setting in settings) {
      settingsMap[setting.name] = setting;
    }

    return ConfigurationSettings(
      id: json['id'].toString(),
      createdAt: json['created'],
      updatedAt: json['updated'],
      settings: settings,
      settingsMap: settingsMap,
    );
  }
}
