import 'dart:math';

import 'package:intl/intl.dart';

class CurrencyNames {
  static const String REC = 'rec';
  static const String EUR = 'eur';
}

class CurrencySymbols {
  static const String REC = 'R';
  static const String EUR = '€';
}

class Currency {
  final String name;
  final String symbol;
  final int scale;
  final int decimals;

  const Currency(
    this.name,
    this.symbol,
    this.scale,
    this.decimals,
  );

  double scaleAmount(num amount) => amountScaled(amount, scale);

  Map<String, dynamic> toJson() {
    return {
      'name': name,
      'symbol': symbol,
      'scale': scale,
      'decimals': decimals,
    };
  }

  static String format(
    num? amount, {
    String symbol = '',
    String? locale = 'es_ES',
    num decimals = 2,
  }) {
    // final formatter = NumberFormat("###.0#", locale);
    final formatter = NumberFormat.currency(
      locale: locale,
      symbol: symbol,
      decimalDigits: decimals as int?,
    );

    // Trim is done to remove trailing space when no symbol provided,
    // messed up tests and added unwanted space
    return amount == null ? '' : formatter.format(amount).trim();
  }

  static const Currency rec = Currency(CurrencyNames.REC, CurrencySymbols.REC, 8, 2);
  static const Currency eur = Currency(CurrencyNames.EUR, CurrencySymbols.EUR, 2, 2);

  static List<Currency> all = [Currency.eur, Currency.rec];

  static Currency find(String? name) {
    return Currency.all.firstWhere(
      (element) => element.name.toUpperCase() == name!.toUpperCase(),
    );
  }

  static double amountScaled(num amount, int scale) {
    return amount / pow(10, scale);
  }
}
