import 'package:http/http.dart';
import 'package:rec_api_dart/src/api_paths.dart';
import 'package:rec_api_dart/src/entities/challenges/challenge.ent.dart';
import 'package:rec_api_dart/src/entities/challenges/challenge_search_data.dart';
import 'package:rec_api_dart/src/env.base.dart';
import 'package:rec_api_dart/src/interceptors/user_token_interceptor.dart';
import 'package:rec_api_dart/src/interfaces/api_list_response.dart';
import 'package:rec_api_dart/src/services/base_service.dart';

/// This service is in charge of all requests related to [Challenges]
///
/// * **Authentication Required** - the user must be authenticatedd
class ChallengesService extends ServiceBase {
  ChallengesService({required EnvBase env, Client? client})
      : super(
          env: env,
          client: client,
          interceptors: [UserTokenInterceptor()],
        );

  /// List all [Challenge]s
  Future<ApiListResponse<Challenge>> list([ChallengeSearchData? data]) {
    final uri = ApiPaths.challenges.withQueryParams(data?.toJson()).asUri(env.API_URL);

    return this.get(uri).then(_mapChallenges);
  }

  /// List all [Challenge] for the current account
  Future<ApiListResponse<Challenge>> getAccountChallenges([ChallengeSearchData? data]) {
    final uri = ApiPaths.accountChallenges.withQueryParams(data?.toJson()).asUri(env.API_URL);

    return this.get(uri).then(_mapChallenges);
  }

  ApiListResponse<Challenge> _mapChallenges(Map<String, dynamic> resp) {
    return ApiListResponse.fromJson(
      resp['data'],
      mapper: (el) => Challenge.fromJson(el),
    );
  }
}
