import 'dart:convert';

import 'package:http/http.dart';
import 'package:rec_api_dart/src/api_paths.dart';
import 'package:rec_api_dart/src/auth.dart';
import 'package:rec_api_dart/src/entities/configuration_settings.ent.dart';
import 'package:rec_api_dart/src/env.base.dart';
import 'package:rec_api_dart/src/grant_types.dart';
import 'package:rec_api_dart/src/services/base_service.dart';

class AppService extends ServiceBase {
  AppService({required EnvBase env, Client? client}) : super(env: env, client: client);

  Future<ConfigurationSettings> getConfigurationSettings() {
    return this.get(ApiPaths.configurationSettings.asUri(env.API_URL)).then(_mapConfig);
  }

  Future getAppToken() async {
    final headers = await getHeaders();
    final body = json.encode({
      'grant_type': GrantTypes.clientCredentials,
      'client_id': env.CLIENT_ID,
      'client_secret': env.CLIENT_SECRET,
    });

    return client
        .post(
          ApiPaths.token.asUri(env.API_URL),
          headers: headers,
          body: body,
        )
        .then((res) => json.decode(res.body))
        .then(saveToken);
  }

  static dynamic saveToken(value) {
    Auth.saveAppTokenData(value);

    return value;
  }

  ConfigurationSettings _mapConfig(Map<String, dynamic> json) {
    return ConfigurationSettings.fromJson(json['data']);
  }
}
