import 'package:http/http.dart';
import 'package:rec_api_dart/src/api_paths.dart';
import 'package:rec_api_dart/src/entities/dto/update_qualification.dart';
import 'package:rec_api_dart/src/entities/qualifications/qualification.ent.dart';
import 'package:rec_api_dart/src/entities/qualifications/qualification_search_data.dart';
import 'package:rec_api_dart/src/env.base.dart';
import 'package:rec_api_dart/src/interceptors/user_token_interceptor.dart';
import 'package:rec_api_dart/src/interfaces/api_list_response.dart';
import 'package:rec_api_dart/src/services/base_service.dart';

/// Service to manage [Qualification]s
class QualificationsService extends ServiceBase {
  QualificationsService({required EnvBase env, Client? client})
      : super(
          env: env,
          client: client,
          interceptors: [UserTokenInterceptor()],
        );

  /// List all [Qualification] for the authenticated user
  Future<ApiListResponse<Qualification>> list(QualificationSearchData? data) {
    final uri = ApiPaths.qualifications.withQueryParams(data?.toJson()).asUri(env.API_URL);

    return this.get(uri).then(_mapQualifications);
  }

  /// Update a qualification instance
  Future updateQualification(String id, UpdateQualificationData data) {
    final uri = ApiPaths.qualifications.append(id).asUri(env.API_URL);

    return this.put(uri, data.toJson());
  }

  ApiListResponse<Qualification> _mapQualifications(Map<String, dynamic> data) {
    return ApiListResponse.fromJson(
      data['data'],
      mapper: (el) => Qualification.fromJson(el),
    );
  }
}
